From ba7dea585c1e60fda702755438de200ed07861b9 Mon Sep 17 00:00:00 2001
From: Michal Demin <michaldemin@gmail.com>
Date: Sun, 22 Apr 2012 14:17:39 +0200
Subject: [PATCH 1/2] Detach the device kernel driver when kernel driver
 loaded.

Signed-off-by: Michal Demin <michaldemin@gmail.com>
---
 src/rtl-sdr.c |   17 +++++++++++++++++
 1 files changed, 17 insertions(+), 0 deletions(-)

diff --git a/src/rtl-sdr.c b/src/rtl-sdr.c
index 8d77347..24e363c 100644
--- a/src/rtl-sdr.c
+++ b/src/rtl-sdr.c
@@ -158,6 +158,7 @@ struct rtlsdr_dev {
 	enum rtlsdr_async_status async_status;
 	rtlsdr_tuner_t *tuner;
 	int rate; /* Hz */
+	int reattach;
 };
 
 #define CRYSTAL_FREQ	28800000
@@ -689,6 +690,15 @@ int rtlsdr_open(rtlsdr_dev_t **out_dev, uint32_t index)
 
 	libusb_free_device_list(list, 0);
 
+	if (libusb_kernel_driver_active(dev->devh, 0)) {
+		// reattach later
+		dev->reattach = 1;
+		if (libusb_detach_kernel_driver(dev->devh, 0)) {
+			fprintf(stderr, "Error detaching kernel driver \n");
+			goto err;
+		}
+	}
+
 	r = libusb_claim_interface(dev->devh, 0);
 	if (r < 0) {
 		fprintf(stderr, "usb_claim_interface error %d\n", r);
@@ -766,6 +776,13 @@ int rtlsdr_close(rtlsdr_dev_t *dev)
 	rtlsdr_deinit_baseband(dev);	
 
 	libusb_release_interface(dev->devh, 0);
+
+	if (dev->reattach) {
+		if (libusb_attach_kernel_driver(dev->devh, 0)) {
+			fprintf(stderr, "Unable to reattach kernel driver\n");
+		}
+	}
+
 	libusb_close(dev->devh);
 
 	libusb_exit(dev->ctx);
-- 
1.7.8.4

